/* Midnight widget (1.1.1) */
/* Created by Wain 21/12/07 */
/* Copyright 2007 Tekuris (www.tekuris.com) */

// Dummy handler for asynchronous system calls

function dummyHandler(param) {}

// Utility

function setHibernationMode(newMode)
{
	widget.system('/usr/bin/pmset force -a hibernatemode ' + newMode, dummyHandler);
}

// Handlers

function handleHibernationValidation(hibernationMode)
{
	// create a string containing the potential hibernation modes
	var validChars = "0123456789";
	
	// check the returned hibernation mode
	if(validChars.indexOf(hibernationMode.outputString.charAt(0)) != -1)
	{
		hibernationAvailable = true;
		
		// check if we should hibernate rather than normal sleep
		if(widget.preferenceForKey(createInstancePreferenceKey("hibernate")) != null)
		{
			document.getElementById("hibernationCheckbox").checked = true;
			
			hibernate = true;
		}
	}
	// otherwise hibernation is not an option
	else
	{
		// disable the checkbox and title
		document.getElementById("hibernationCheckbox").disabled = true;
		
		// null the preference
		widget.setPreferenceForKey(null, createInstancePreferenceKey("hibernate"));
	}
}

function handleAirportValidation(airportProfile)
{
	// check the length of the returned message, will be empty if no airport present
	if(airportProfile.outputString.length >= 10)
	{
		airportAvailable = true;
		
		// check if we should disable the airport
		if(widget.preferenceForKey(createInstancePreferenceKey("disableAirport")) != null)
		{
			document.getElementById("airportCheckbox").checked = true;
			
			disableAirport = true;
		}
	}
	// otherwise disabling the airport is not an option
	else
	{
		// disable the checkbox and title
		document.getElementById("airportCheckbox").disabled = true;
		
		// null the preference
		widget.setPreferenceForKey(null, createInstancePreferenceKey("disableAirport"));
	}
}

function handleVersionNumber(versionNumber)
{
	var version = versionNumber.outputString;
	
	document.getElementById("versionText").innerText = version;
}

function handleCheckAndDisableAirport(airportEnabled)
{
	if(airportEnabled.outputString.indexOf("On") != -1)
	{
		// turn airport off
		// this command really wants 'sudo' but works without (cp permission error raised)
		widget.system('/usr/sbin/networksetup -setairportpower off', dummyHandler);
		
		// start a timer to reenable airport after waking
		setTimeout('widget.system("/usr/sbin/networksetup -setairportpower on", dummyHandler);', 30000);
	}
}

function handleHibernationAndSleep(resetMode)
{
	// add the hibernation option if requested
	if(hibernate)
	{
		// force the hibernation mode
		setHibernationMode(1);
		
		// start a timer to reset the hibernation mode after waking
		setTimeout('resetHibernationMode(' + resetMode.outputString.charAt(0) + ');', 25000);
	}
	
	// send the sleep command
	setTimeout('sleepFinder();', 500);
}

function sleepFinder()
{
    // reset the busy flag if hibernation is not requested
	if(!hibernate)
    {
        busy = false;
    }

	// tell the finder to sleep
	widget.system("/usr/bin/osascript -e 'tell application \"Finder\" to sleep'", dummyHandler);	
}

function resetHibernationMode(resetMode)
{
	if(hibernationAvailable)
	{		
		// reset the hibernation mode
		setHibernationMode(resetMode);
	}
	
	busy = false;
}

// Main

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    setupParts();
	
	// perform setup and load the preferences if we can get the widget window
	if(window.widget)
	{
		// create some flags to help manage our state
		busy = false;
		
		fusToLogin = false;
		hibernate = false;
		hibernationAvailable = false;
		disableAirport = false;
		airportAvailable = false;
		
		// check if we should switch to the login screen
		if(widget.preferenceForKey(createInstancePreferenceKey("fusToLogin")) != null)
		{
			document.getElementById("fusCheckbox").checked = true;
			
			fusToLogin = true;
		}
		
		// make sure hibernation is available on this Mac
		widget.system("/usr/bin/pmset -g | /usr/bin/awk '/hibernatemode/ {print $2}'", handleHibernationValidation);
		
		// make sure this Mac has an airport
		widget.system('/usr/sbin/system_profiler SPAirPortDataType', handleAirportValidation);
				
		// get our name
		var name = widget.preferenceForKey(createInstancePreferenceKey("name"));
		if(name != null)
		{
			document.getElementById("nameTextIn").value = name;
			document.getElementById("nameTextOut").innerText = name;
		}
		else
		{
			document.getElementById("nameTextIn").value = getLocalizedString("Untitled");
		}
		
		// get the current version number
		widget.system('/bin/sh -c "defaults read `pwd`/Info CFBundleVersion"', handleVersionNumber);
	}
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

// Front events

function mouseOver(event)
{
	// fade the sleeping finder image in
	var itemToFadeIn = document.getElementById("FinderAsleep");
	var fadeHandler = function(a, c, s, f){ itemToFadeIn.style.opacity = c; };
	new AppleAnimator(250, 10, 0.0, 1.0, fadeHandler).start();
}

function mouseOut(event)
{
	// fade the sleeping finder image out
	var itemToFadeOut = document.getElementById("FinderAsleep");
	var fadeHandler = function(a, c, s, f){ itemToFadeOut.style.opacity = c; };
	new AppleAnimator(250, 10, 1.0, 0.0, fadeHandler).start();
}

// activate the widget
function activate(event)
{
	// if we are currently busy we will ignore the request
	if(busy)
	{
		return;
	}
	
	// set the flag to show that we are busy
	busy = true;
	
	// check if we should disable airport
	if(disableAirport)
	{
		// check the network setup to see if airport power is 'On'
		widget.system('/usr/sbin/networksetup -getairportpower', handleCheckAndDisableAirport);
	}
	
	// next fus to login if requested
	if(fusToLogin)
	{
		widget.system('"/System/Library/CoreServices/Menu Extras/User.menu/Contents/Resources/CGSession" -suspend', dummyHandler);
	}
	
	// wait a short time for things to settle then sleep, passing the hibernation mode to reset to
	setTimeout('widget.system("/usr/bin/pmset -g | /usr/bin/awk \'/hibernatemode/ {print $2}\'", handleHibernationAndSleep);', 3000);
}

// Back events

// when the fus checkbox is changed
function setFusLogin(element)
{
	// store the preferences if we can get the widget window
	if(window.widget)
	{
		if(element.checked)
		{
			widget.setPreferenceForKey(true, createInstancePreferenceKey("fusToLogin"));
			
			fusToLogin = true;
		}
		else
		{
			widget.setPreferenceForKey(null, createInstancePreferenceKey("fusToLogin"));
			
			fusToLogin = false;
		}
	}
}
// when the fus label is clicked
function fusLoginLabelClicked(event)
{
	// toggle the state of the fus checkbox
	var element = document.getElementById('fusCheckbox');
	element.checked = !element.checked;
	
	// call the fus change function
	setFusLogin(element);
}

// when the hibernate checkbox is changed
function setHibernation(element)
{
	// make sure hibernation is enabled on this Mac
	if(hibernationAvailable)
	{
		// load the preferences if we can get the widget window
		if(window.widget)
		{
			if(element.checked)
			{
				widget.setPreferenceForKey(true, createInstancePreferenceKey("hibernate"));
				
				hibernate = true;
			}
			else
			{
				widget.setPreferenceForKey(null, createInstancePreferenceKey("hibernate"));
				
				hibernate = false;
			}
		}
	}
}
// when the hibernate label is clicked
function hibernateLabelClicked(event)
{
	// make sure hibernation is enabled on this Mac
	if(hibernationAvailable)
	{
		// toggle the state of the hibernate checkbox
		var element = document.getElementById('hibernationCheckbox');
		element.checked = !element.checked;
		
		// call the hibernate change function
		setHibernation(element);
	}
}

// when the airport checkbox is changed
function setDisableAirport(element)
{
	// make sure airport is available on this Mac
	if(airportAvailable)
	{
		// load the preferences if we can get the widget window
		if(window.widget)
		{
			if(element.checked)
			{
				widget.setPreferenceForKey(true, createInstancePreferenceKey("disableAirport"));
				
				disableAirport = true;
			}
			else
			{
				widget.setPreferenceForKey(null, createInstancePreferenceKey("disableAirport"));
				
				disableAirport = false;
			}
		}
	}
}
// when the airport label is clicked
function airportLabelClicked(event)
{
	// make sure hibernation is enabled on this Mac
	if(airportAvailable)
	{
		// toggle the state of the airport checkbox
		var element = document.getElementById('airportCheckbox');
		element.checked = !element.checked;
		
		// call the airport change function
		setDisableAirport(element);
	}
}

function nameFocused(event)
{
	// set a timeout to select the current name
	setTimeout(function() {document.getElementById("nameTextIn").select();}, 50);
}
function nameChanged(event)
{
	var newName = document.getElementById("nameTextIn").value;
	
	document.getElementById("nameTextOut").innerText = newName;
	
	widget.setPreferenceForKey(newName, createInstancePreferenceKey("name"));
}

function linkToMidnight(event)
{
	widget.openURL('http://www.tekuris.com/products/midnight');
}
